#include <stdio.h>
#include <math.h>
#include <avs/flow.h> 
#include <avs/avs_data.h>
#include <avs/field.h>

static FILE *fp = NULL;
#define Default_fname "stock.out"

AVSinit_modules()
{
  int merge();

  AVSmodule_from_desc(merge);
}

/*  The routine "merge" is the description routine.  */
merge()
{
  int merge_compute();	 /* declare the compute function (below) */
  int in_port,out_port;	 /* temporary to hold the port number */
  int parm;		 /* temporary to hold the parm number */

  /* Set the module name and type */
  AVSset_module_name("file_out_DEC", MODULE_MAPPER);
  
  /* Create an input port for the required field input */
  in_port = AVScreate_input_port("BS_bias",  "real",REQUIRED);   
  in_port = AVScreate_input_port("AMC_bias", "real",REQUIRED);   
  in_port = AVScreate_input_port("EUS_bias", "real",REQUIRED);   
  in_port = AVScreate_input_port("AMS_bias", "real",REQUIRED);   

  /* Create output ports for the result */
  out_port = AVScreate_output_port("Field Output", 
				   "field 1D scalar uniform float");

  /* Add parameter:  */
  parm = AVSadd_parameter("Output Model Result Browser", "string", Default_fname, 0, 0);
  AVSconnect_widget(parm,"browser");
  
  /* Tell avs what subroutine to call to do the compute */
  AVSset_compute_proc(merge_compute);
}

/*  merge_compute is the compute routine.  */
merge_compute(bs_bias,amc_bias,eus_bias,ams_bias,output,OutFileName)
AVSfield_float **output;
char *OutFileName;
float *bs_bias,*amc_bias,*eus_bias,*ams_bias;
{
  AVSfield_float *temp_field = NULL;
  int dims[1];
  double fabs();

  /* Allocate space for output */
  dims[0] = 4;
  temp_field = (AVSfield_float *)AVSdata_alloc
    ("field 1D scalar uniform float",dims);
  /*  if (temp_field == NULL) {
    AVSerror("Allocation of output field failed.");
    return(0);
    } */
     
  if (*output) {
    temp_field->data[0] = fabs(*bs_bias);
    temp_field->data[1] = fabs(*amc_bias);
    temp_field->data[2] = fabs(*eus_bias);
    temp_field->data[3] = fabs(*ams_bias);
    AVSdata_free("field",*output);
  }

  *output = temp_field;
  /*  fprintf(stderr,"\nB-S =%8.5f AMC =%8.5f EUS =%8.5f AMS =%8.5f\n",
   *bs_bias,*amc_bias,*eus_bias,*ams_bias);  */

  if (AVSparameter_changed("Output Model Result Browser")||(fp == NULL)) {
    fprintf(stderr,"open file %s\n",OutFileName);
    if (fp != NULL) fclose(fp);
    if ((fp = fopen(OutFileName, "w")) == NULL) {
      AVSerror("finance_out: cannot open %s\n",OutFileName);
      return(1);
    }
    fprintf(fp,"B-S bias AMC bias EUS bias AMS bias\n");
  }
  fprintf(fp,"%8.5f %8.5f %8.5f %8.5f\n",*bs_bias,*amc_bias,*eus_bias,*ams_bias);  

  /* When we're done, we return 1 to indicate success */
  return(1);
}

